#ifndef RAJZOLAS_H_INCLUDED
#define RAJZOLAS_H_INCLUDED

#include <cstdint>
#include <string>
#include <vector>

#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#include "point.hpp"

struct Color
{
    uint8_t r, g, b, a;

    Color() :
        r(0x00), g(0x00), b(0x00), a(0xFF) {
        }

    Color(uint8_t r, uint8_t g, uint8_t b, uint8_t a = 0xFF) :
        r(r), g(g), b(b), a(a)
    { }

    explicit Color(uint32_t c) :
        r((c & 0xFF000000) >> 24),
        g((c & 0x00FF0000) >> 16),
        b((c & 0x0000FF00) >> 8),
        a((c & 0x000000FF) >> 0)
    { }

    uint32_t convert() const
    {
        return  r << 24 |
                g << 16 |
                b << 8  |
                a;
    }
};

template <typename T = void>
struct MySDL {
    static SDL_Surface *screen;
};

template <typename T>
SDL_Surface *MySDL<T>::screen;

inline void sdl_init(const char* title, Point windowsize) {
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
    MySDL<>::screen = SDL_SetVideoMode(windowsize.x, windowsize.y, 32, 0);
    SDL_WM_SetCaption(title, title);
}

inline void sdl_rajzol() {
    SDL_Flip(MySDL<>::screen);
}

inline void sdl_vege() {
    bool quit = false;
    while (!quit) {
        SDL_Event event;
        SDL_WaitEvent(&event);

        switch (event.type) {
            case SDL_QUIT:
            case SDL_KEYDOWN:
                quit = true;
                break;
            default:
                break;
        }
    }

    SDL_Quit();
}

inline void sdl_var(int ms) {
    SDL_Delay(ms);
}

inline void sdl_torol() {
    SDL_FillRect(MySDL<>::screen, NULL, 0);
}

inline void teglalap_rajzol(Point p1, Point p2, Color c) {
    boxColor(MySDL<>::screen, p1.x, p1.y, p2.x, p2.y, c.convert());
}


inline void kor_rajzol(Point k, int atmero, Color c) {
    filledCircleColor(MySDL<>::screen, k.x, k.y, atmero, c.convert());
}

inline void kerekitett_teglalap_rajzol(Point p1, Point p2, int sugar, Color c) {
    roundedBoxColor(MySDL<>::screen, p1.x, p1.y, p2.x, p2.y, sugar, c.convert());
}

inline void szakasz_rajzol(Point p1, Point p2, int vastagsag, Color c) {
    thickLineColor(MySDL<>::screen, p1.x, p1.y, p2.x, p2.y, vastagsag, c.convert());
}

inline void szoveg_rajzol(Point p, std::string const& szoveg, Color c) {
    stringColor(MySDL<>::screen, p.x, p.y, szoveg.c_str(), c.convert());
}

inline void sokszog_rajzol(std::vector<Point> const& pontok, Color c) {
    int n = pontok.size();
    int16_t * px = new int16_t[n];
    int16_t * py = new int16_t[n];

    for(int i = 0; i < n; ++i) {
        px[i] = pontok[i].x;
        py[i] = pontok[i].y;
    }

    filledPolygonColor(MySDL<>::screen, px, py, n, c.convert());

    delete[] py;
    delete[] px;
}

#endif // RAJZOLAS_H_INCLUDED
